'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _omit2 = require('lodash/omit');

var _omit3 = _interopRequireDefault(_omit2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _constants = require('../constants');

var _optimisticRedux = require('optimistic-redux');

var _optimisticRedux2 = _interopRequireDefault(_optimisticRedux);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * A double reducer enhancer that wraps a reducer with the
 * request handling logic needed to keep track of requests
 * as well as an optimistic reducer enhancer provided by
 * redux-optimist.
 */
exports.default = function (reducer) {
    return (0, _optimisticRedux2.default)(function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var action = arguments[1];

        var resolve = function resolve(nextRequestState) {
            return _extends({}, reducer((0, _omit3.default)(state, ["requests"]), action), {
                requests: nextRequestState
            });
        };

        var type = action.type,
            payload = action.payload;


        switch (type) {
            case _constants.BEGIN_REQUEST:
                {
                    return resolve(_extends({}, state.requests || {}, _defineProperty({}, payload.requestId, {
                        id: payload.requestId,
                        status: "pending",
                        type: _constants.BEGIN_REQUEST,
                        name: payload.requestName,
                        timestamp: Date.now()
                    })));
                }
            case _constants.COMPLETE_REQUEST:
                {
                    return resolve(_extends({}, state.requests, _defineProperty({}, payload.requestId, _extends({}, state.requests[payload.requestId], {
                        status: "complete",
                        type: _constants.COMPLETE_REQUEST
                    }))));
                }
            case _constants.FAIL_REQUEST:
                {
                    return resolve(_extends({}, state.requests, _defineProperty({}, payload.requestId, _extends({}, state.requests[payload.requestId], {
                        status: "failed",
                        type: _constants.FAIL_REQUEST,
                        error: payload.error
                    }))));
                }
        }

        return resolve(state.requests || {});
    });
};